﻿Imports System.IO
Imports System.Xml
Imports System.Xml.Serialization

Public Class FormFlareBatchCommandsManager

Private Sub ButtonSelectRootFolder_Click(ByVal sender As System.Object,
																				 ByVal e As System.EventArgs) _
																 Handles ButtonSelectRootFolder.Click
	'Populate a label for the root folder with a selection from a file dialog.
	'The root folder is where the targets will be found.
	'Includes subfolders.
	If FolderBrowserDialogRootFolder.ShowDialog() = DialogResult.OK Then
		LabelRootFolderPath.Text = FolderBrowserDialogRootFolder.SelectedPath
		RefreshGrid(LabelRootFolderPath.Text)
	End If
End Sub

Private Sub ButtonFlareAppPath_Click(ByVal sender As System.Object,
																		 ByVal e As System.EventArgs) _
														 Handles ButtonFlareAppPath.Click
	'Populate a label for the path to Flare.app. 
	'This is how the manager determines the location of madbuild.exe.
	If FolderBrowserDialogFlareAppPath.ShowDialog() = DialogResult.OK Then
		LabelFlareAppPath.Text = FolderBrowserDialogFlareAppPath.SelectedPath
	End If
End Sub

Private Sub ButtonBATFilesFolder_Click(ByVal sender As System.Object,
																			 ByVal e As System.EventArgs) _
															 Handles ButtonBATFilesFolder.Click
	'Populate a label for the path to the folder for the generated BAT files.
	If FolderBrowserDialogBATFilesFolder.ShowDialog() = DialogResult.OK Then
		LabelBATFilesFolder.Text = FolderBrowserDialogBATFilesFolder.SelectedPath
	End If
End Sub

Private Sub ButtonTFPath_Click(ByVal sender As System.Object,
															 ByVal e As System.EventArgs) _
											 Handles ButtonTFPath.Click
	'Populate a label for the path to the folder that contains TF.exe, 
	'the command executable for Team Foundation Server.
	'This is to support adding get commands for TFS.
	If FolderBrowserDialogTFPath.ShowDialog() = DialogResult.OK Then
		LabelTFPath.Text = FolderBrowserDialogTFPath.SelectedPath
	End If
End Sub

Private Sub RefreshGrid(ByVal rootPath As String)
	'Clear the grid and populate it with all of the targets in the folder.
	DataGridViewTargets.Rows.Clear()
	For Each foundFile _
		As String In My.Computer.FileSystem.GetFiles(rootPath,
			Microsoft.VisualBasic.FileIO.SearchOption.SearchAllSubDirectories,
			"*.fltar")
		'This if excludes the prefix we use for template targets.
		If Not Path.GetFileName(foundFile).Contains("ztemplate") Then
			Dim directoryName = Path.GetDirectoryName(foundFile)
			'finding the Targets folder part in the path.
			Dim projTarg As String
			projTarg = "Project\Targets"
			Dim indexForProjTarg = directoryName.IndexOf(projTarg)
			directoryName = directoryName.Substring(0, indexForProjTarg)
			Dim targetNameWithFolder As String
			targetNameWithFolder = _
				Path.GetDirectoryName(foundFile).Substring(indexForProjTarg + _
					15).Replace("\", "/") _
					+ "/" + Path.GetFileName(foundFile)
			If targetNameWithFolder.Substring(0, 1) = "/" Then
				targetNameWithFolder = targetNameWithFolder.Substring(1)
			End If
			Dim projectFileName = ""
			'Find the project file for the target.
			'Depending on how flexible Flare gets with the file locations, 
			'this may need to be revisited.
			For Each foundProjectFile _
				As String In My.Computer.FileSystem.GetFiles(directoryName,
					Microsoft.VisualBasic.FileIO.SearchOption.SearchTopLevelOnly,
					"*.flprj")
				projectFileName = foundProjectFile
			Next
			'Add a row to the grid for the target.
			DataGridViewTargets.Rows.Add(True, Path.GetFileName(foundFile),
																	 projectFileName,
																	 ReturnType(foundFile).ToString,
																	 targetNameWithFolder)
		End If
	Next
End Sub

Private Sub FormFlareBatchCommandsManager_Load(ByVal sender As System.Object,
																							 ByVal e As System.EventArgs) _
																			 Handles MyBase.Load
	'Populate the grid on load of the form.
	RefreshGrid(My.Settings.RootFolderPath)
End Sub

Private Sub ButtonGetNow_Click(ByVal sender As System.Object,
															 ByVal e As System.EventArgs) _
											 Handles ButtonGetNow.Click
	'Get latest using TF.exe. Gets for the root folder on the form.
	Dim getLines As String
	getLines = vbCrLf + "cd " + LabelTFPath.Text.ToString + vbCrLf
	getLines = getLines + "TF.exe get " + LabelRootFolderPath.Text.ToString +
						 " /recursive" + vbCrLf
	getLines = getLines + "Pause"
	Dim file As System.IO.FileStream
	Dim batchPath = LabelBATFilesFolder.Text.ToString + "\" + "TFGet.bat"
	file = System.IO.File.Create(batchPath)
	file.Close()
	My.Computer.FileSystem.WriteAllText(batchPath, getLines, True,
																			New System.Text.UTF8Encoding(True))
	Shell(batchPath)
End Sub

Private Function getOutputFolder(ByVal targetFile As String)
	'Return the attribute value for OutputFolder in the CatapultTarget
	'element (root) or if there is not one, return the text in the label
	'for the folder to which to output the batch files
	Dim targetOutputFolder As String
	Dim flareTargetXHTML As XDocument = XDocument.Load(targetFile)
	If Not flareTargetXHTML.Root.Attribute("OutputFolder") Is Nothing Then
		targetOutputFolder = flareTargetXHTML.Root.Attribute("OutputFolder").Value
		Return targetOutputFolder
	Else
		Return LabelBATFilesFolder.Text
	End If
End Function

Private Sub CreateMultipleBatchFiles()
	'This creates many files that each build one target.

	'Generate a batch file for every target in the grid rows.
	For Each row As DataGridViewRow In DataGridViewTargets.Rows
		If Not row.Cells(1).Value = "" Then
			'Create a BAT file using the project and target name.
			Dim file As System.IO.FileStream
			Dim batchPath As String
			Dim fileNameFragment As String
			fileNameFragment = row.Cells(1).FormattedValue.ToString
			Dim targetFileName = fileNameFragment.Substring(0,
													 fileNameFragment.Length - 6)
			fileNameFragment = "_" + _
				fileNameFragment.Substring(0, fileNameFragment.Length - 6)

			Dim fileNameFragment2 As String
			fileNameFragment2 = Path.GetFileName(row.Cells(2).FormattedValue.ToString)
			fileNameFragment2 = "_" + _
				fileNameFragment2.Substring(0, fileNameFragment2.Length - 6)
			fileNameFragment = fileNameFragment2 + fileNameFragment
			batchPath = LabelBATFilesFolder.Text.ToString + "\" +
					"FlareTargetBatch" + fileNameFragment + ".bat"
			file = System.IO.File.Create(batchPath)
			file.Close()
			Dim comment As String
			comment = "REM comment goes here" + vbCrLf
			My.Computer.FileSystem.WriteAllText(batchPath,
															comment, True, New System.Text.UTF8Encoding(False))

			'Add a line to the batch file to change the directory to the TF.exe path
			'and a line to run TF.exe get on the root folder.
			If CheckBoxGetLatestVersion.Checked Then
				Dim cdString As String
				cdString = "cd " + LabelTFPath.Text.ToString + vbCrLf
				My.Computer.FileSystem.WriteAllText(batchPath, cdString, True)
				Dim getPath As String
				getPath = "TF.exe get " + LabelRootFolderPath.Text.ToString +
					" /recursive" + vbCrLf
				My.Computer.FileSystem.WriteAllText(batchPath, getPath, True)
			End If

			'Add a line to the batch file to change the directory to the
			'madbuild.exe path and a command line for the target in the row.
			'Include -log true if selected.
			Dim flareAppPath As String
			flareAppPath = "cd " + LabelFlareAppPath.Text.ToString + vbCrLf
			My.Computer.FileSystem.WriteAllText(batchPath, flareAppPath, True)

			Dim buildLine As String
			buildLine = "madbuild -project " + row.Cells(2).FormattedValue.ToString +
									" -target " + row.Cells(4).FormattedValue.ToString

			If CheckBoxAddLogTrue.Checked = True Then
				buildLine = buildLine + " -log true"
			End If

			buildLine = buildLine + vbCrLf

			If row.Cells(0).Value = True Then
				My.Computer.FileSystem.WriteAllText(batchPath, buildLine, True)
			Else
				buildLine = "rem " + buildLine
				My.Computer.FileSystem.WriteAllText(batchPath, buildLine, True)
			End If

			'Add lines to place the output in a ZIP file.
			If CheckBoxCreateZIP.Checked Then
				My.Computer.FileSystem.WriteAllText(batchPath, "cd " + _
					Label7ZIPPath.Text + vbCrLf, True)
				Dim folderToArchive As String
				Dim pathOfTarget As String
				pathOfTarget = row.Cells(4).FormattedValue.ToString
				folderToArchive = getOutputFolder(pathOfTarget)
				Dim ZIPCommand As String
				ZIPCommand =
					 "REM z target is ZIP filename and source is the path after it" + _
					 vbCrLf + "7z a C:\FlareOutput\zip\" + fileNameFragment + _
					 ".zip " + folderToArchive + "\" + _
					 "Output\" + targetFileName + "\*" + vbCrLf
				My.Computer.FileSystem.WriteAllText(batchPath, ZIPCommand, True)
			End If

			If CheckBoxAddPause.Checked Then
				My.Computer.FileSystem.WriteAllText(batchPath, "Pause", True)
			End If
		End If
	Next

End Sub

Private Sub CreateSingleBatchFile(ByVal fileNameFragment As String)
	'This creates one file that builds many targets.
	'Create a BAT file in the folder indicated on the form.
	'The name is FlareTargetBatch + the fragment passed to this procedure.
	Dim file As System.IO.FileStream
	Dim batchPath As String
	batchPath = LabelBATFilesFolder.Text.ToString + _
		"\" + "FlareTargetBatch" + fileNameFragment + ".bat"
	file = System.IO.File.Create(batchPath)
	file.Close()
	Dim comment As String
	comment = "REM comment goes here" + vbCrLf
	My.Computer.FileSystem.WriteAllText(batchPath,
		comment, True, New System.Text.UTF8Encoding(False))

	'Add a line to the batch file to change the directory
	'to the TF.exe path
	'and a line to run TF.exe get on the root folder.
	If CheckBoxGetLatestVersion.Checked Then
		Dim cdString As String
		cdString = "cd " + LabelTFPath.Text.ToString + vbCrLf
		My.Computer.FileSystem.WriteAllText(batchPath, cdString, True)
		Dim getPath As String
		getPath = "TF.exe get " + LabelRootFolderPath.Text.ToString + _
			" /recursive" + vbCrLf
		My.Computer.FileSystem.WriteAllText(batchPath, getPath, True)
	End If

	'Add a line to the batch file to change the directory
	'to the madbuild.exe path.

	Dim flareAppPath As String
	flareAppPath = "cd " + LabelFlareAppPath.Text.ToString + vbCrLf
	My.Computer.FileSystem.WriteAllText(batchPath, flareAppPath, True)

	'Add a command line for every target in the grid.
	'Include -log true if selected.
	For Each row As DataGridViewRow In DataGridViewTargets.Rows
		Dim buildLine As String
		buildLine = "madbuild -project " +
				row.Cells(2).FormattedValue.ToString +
				" -target " + row.Cells(1).FormattedValue.ToString
		If CheckBoxAddLogTrue.Checked = True Then
			buildLine = buildLine + " -log true"
		End If
		buildLine = buildLine + vbCrLf

		If row.Cells(0).Value = True Then
			My.Computer.FileSystem.WriteAllText(batchPath, buildLine, True)
		Else
			buildLine = "rem " + buildLine
			My.Computer.FileSystem.WriteAllText(batchPath, buildLine, True)
		End If
	Next

	If CheckBoxAddPause.Checked Then
		My.Computer.FileSystem.WriteAllText(batchPath, "Pause", True)
	End If
End Sub



Private Sub ButtonCreateSingleBatchFile_Click(ByVal sender As System.Object,
																							ByVal e As System.EventArgs) _
																	 Handles ButtonCreateSingleBatchFile.Click
	CreateSingleBatchFile(System.Guid.NewGuid.ToString)
End Sub

Private Sub ButtonCreateBatchFileForEach_Click(ByVal sender As System.Object,
																							 ByVal e As System.EventArgs) _
																	 Handles ButtonCreateBatchFileForEach.Click
	CreateMultipleBatchFiles()
End Sub

Private Function ReturnType(ByVal targetFile As String)
	'Return the attribute value for Type in the CatapultTarget element (root)
	Dim targetType As String
	Dim flareTargetXHTML As XDocument = XDocument.Load(targetFile)
	targetType = flareTargetXHTML.Root.Attribute("Type").Value
	Return targetType
End Function

Private Sub CheckBoxTFS_CheckedChanged(ByVal sender As System.Object,
																			 ByVal e As System.EventArgs) _
															 Handles CheckBoxTFS.CheckedChanged
	'This doesn't provide much value at this point.
	'If there are other TFS options, this will enable or disable those.
	If CheckBoxTFS.Checked = True Then
		CheckBoxGetLatestVersion.Enabled = True
	Else
		CheckBoxGetLatestVersion.Enabled = False
		CheckBoxGetLatestVersion.Checked = False
	End If
End Sub

Private Sub ButtonSaveConfiguration_Click(ByVal sender As System.Object,
																					ByVal e As System.EventArgs) _
																	Handles ButtonSaveConfiguration.Click
	'Create an XML file to store the manager's configuration.
	Dim configFlareBatchXML As XDocument = _
<?xml version="1.0" encoding="utf-8" standalone="yes"?>
<!--Flare Batch Commands Manager Configuration File-->
<FlareBatchCommandsConfiguration>

</FlareBatchCommandsConfiguration>

	'Add elements to store the settings in the manager and
	'populate the elements with values from the form.
	Dim RootFolder As XElement
	RootFolder = <RootFolder></RootFolder>
	RootFolder.Value = LabelRootFolderPath.Text
	configFlareBatchXML.Root.Add(RootFolder)
	Dim FlareApp As XElement
	FlareApp = <FlareApp></FlareApp>
	FlareApp.Value = LabelFlareAppPath.Text
	configFlareBatchXML.Root.Add(FlareApp)
	Dim BATFiles As XElement
	BATFiles = <BATFiles></BATFiles>
	BATFiles.Value = LabelBATFilesFolder.Text
	configFlareBatchXML.Root.Add(BATFiles)
	Dim TFSControlled As XElement
	TFSControlled = <TFSControlled></TFSControlled>
	TFSControlled.Value = CheckBoxTFS.Checked.ToString
	configFlareBatchXML.Root.Add(TFSControlled)
	Dim TFSGetLatest As XElement
	TFSGetLatest = <TFSGetLatest></TFSGetLatest>
	TFSGetLatest.Value = CheckBoxGetLatestVersion.Checked.ToString
	configFlareBatchXML.Root.Add(TFSGetLatest)
	Dim TFEXEPath As XElement
	TFEXEPath = <TFEXEPath></TFEXEPath>
	TFEXEPath.Value = LabelTFPath.Text
	configFlareBatchXML.Root.Add(TFEXEPath)
	Dim AddLogTrue As XElement
	AddLogTrue = <AddLogTrue></AddLogTrue>
	AddLogTrue.Value = CheckBoxAddLogTrue.Checked.ToString
	configFlareBatchXML.Root.Add(AddLogTrue)
	Dim AddPause As XElement
	AddPause = <AddPause></AddPause>
	AddPause.Value = CheckBoxAddPause.Checked.ToString
	configFlareBatchXML.Root.Add(AddPause)
	Dim CreateZIP As XElement
	CreateZIP = <ZIPPath></ZIPPath>
	CreateZIP.Value = CheckBoxCreateZIP.Checked.ToString
	configFlareBatchXML.Root.Add(CreateZIP)
	Dim ZIPPath As XElement
	ZIPPath = <ZIPPath></ZIPPath>
	ZIPPath.Value = Label7ZIPPath.Text
	configFlareBatchXML.Root.Add(ZIPPath)
	'This one has children since it represents the grid on the form.
	'Only store the Include setting, target,
	'and project (first three columns)
	'since this is all we need to identify.
	Dim Commands As XElement
	Commands = <Commands></Commands>
	'Loop through each row in the grid and create an element and child elements.
	For Each row In DataGridViewTargets.Rows
		Dim newRow As XElement
		Dim includeCell As XElement
		Dim targetCell As XElement
		Dim projectCell As XElement
		newRow = <CommandInfo></CommandInfo>
		includeCell = <Include></Include>
		includeCell.Name = "Include"
		includeCell.Value = row.Cells(0).FormattedValue.ToString
		newRow.Add(includeCell)
		targetCell = <Target></Target>
		targetCell.Name = "Target"
		targetCell.Value = row.Cells(4).FormattedValue.ToString
		newRow.Add(targetCell)
		projectCell = <Project></Project>
		projectCell.Name = "Project"
		projectCell.Value = row.Cells(2).FormattedValue.ToString
		newRow.Add(projectCell)
		Commands.Add(newRow)
	Next
	configFlareBatchXML.Root.Add(Commands)
	'Save the configuration file. Limit the dialog to XML.
	SaveFileDialogConfig.FileName = LabelBATFilesFolder.Text +
																	"\SpecialConfiguration.xml"
	SaveFileDialogConfig.Filter = "XML (*.xml)|*.xml"
	If SaveFileDialogConfig.ShowDialog() = DialogResult.OK Then
		configFlareBatchXML.Save(SaveFileDialogConfig.FileName)
	End If
End Sub

Private Sub ButtonLoadConfiguration_Click(ByVal sender As System.Object,
																					ByVal e As System.EventArgs) _
																	Handles ButtonLoadConfiguration.Click
	'Open the dialog to select the configuration file.
	OpenFileDialogConfig.Filter = "XML (*.xml)|*.xml"

	'Loop through the elements to populate the form.
	'The grid will be populated by the selection of RootFolder.
	'The settings for Include come from the file.
	If OpenFileDialogConfig.ShowDialog() = DialogResult.OK Then
		Dim configFlareBatchXML As XDocument =
				XDocument.Load(OpenFileDialogConfig.FileName)
		For Each element In configFlareBatchXML.Root.Elements

			If element.Name = "RootFolder" Then
				LabelRootFolderPath.Text = element.Value
			ElseIf element.Name = "FlareApp" Then
				LabelFlareAppPath.Text = element.Value
			ElseIf element.Name = "BATFiles" Then
				LabelBATFilesFolder.Text = element.Value
			ElseIf element.Name = "TFEXEPath" Then
				LabelTFPath.Text = element.Value
			ElseIf element.Name = "ZIPPath" Then
				Label7ZIPPath.Text = element.Value
			End If

			If element.Name = "TFSControlled" Then
				CheckBoxTFS.Checked = element.Value
			End If

			If element.Name = "TFSGetLatest" Then
				CheckBoxGetLatestVersion.Checked = element.Value
			End If

			If element.Name = "AddPause" Then
				CheckBoxAddPause.Checked = element.Value
			End If

			If element.Name = "CreateZIP" Then
				CheckBoxCreateZIP.Checked = element.Value
			End If

			If element.Name = "AddLogTrue" Then
				CheckBoxAddLogTrue.Checked = element.Value
			End If

			If element.Name = "Commands" Then
				For Each element2 In element.Descendants
					For Each row In DataGridViewTargets.Rows
						If row.Cells(1).FormattedValue.ToString = _
								element2.Descendants("Target").Value And
							row.Cells(2).FormattedValue.ToString = _
								element2.Descendants("Project").Value Then
							row.Cells(0).Value = element2.Descendants("Include").Value
						End If
					Next
				Next
			End If
		Next
	End If
End Sub

End Class

